from numba import cuda
import numpy as np
import os
import time

# Check the GPU assigned by CUDA
device = cuda.get_current_device()
print(f"Running on device: {device.name}")

# Define kernel
@cuda.jit
def matrix_mult(A, B, C):
    row, col = cuda.grid(2)
    if row < C.shape[0] and col < C.shape[1]:
        tmp = 0
        for k in range(A.shape[1]):
            tmp += A[row, k] * B[k, col]
        C[row, col] = tmp

# Initialize matrices
N = 4096  # Small matrix size
A = np.random.rand(N, N).astype(np.float32)
B = np.random.rand(N, N).astype(np.float32)
C = np.zeros((N, N), dtype=np.float32)

# Transfer data to GPU
A_device = cuda.to_device(A)
B_device = cuda.to_device(B)
C_device = cuda.device_array((N, N), dtype=np.float32)

# Define thread/block layout
threads_per_block = (16, 16)
blocks_per_grid_x = int(np.ceil(C.shape[0] / threads_per_block[0]))
blocks_per_grid_y = int(np.ceil(C.shape[1] / threads_per_block[1]))
blocks_per_grid = (blocks_per_grid_x, blocks_per_grid_y)

# Measure start time
start_time = time.time()

# Launch the kernel
matrix_mult[blocks_per_grid, threads_per_block](A_device, B_device, C_device)

# Copy result back to host
C = C_device.copy_to_host()
print("Matrix multiplication result:")
print(C)

# Wait for the GPU to finish
cuda.synchronize()

# Measure end time
end_time = time.time()

# Print elapsed time
print(f"Time taken for GPU computation: {end_time - start_time:.6f} seconds")

